import logging
import json

from django.core.exceptions import ImproperlyConfigured
from django.utils import six

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from .exceptions import RateBackendError
from .models import Rate
from .settings import EXCHANGE_APP_ID, EXCHANGE_URL, EXCHANGE_BASE_CURRENCY

logger = logging.getLogger(__name__)


class RateBackend:
    source_name = "openexchange.org"

    def __init__(self):
        if not EXCHANGE_APP_ID:
            raise ImproperlyConfigured(
                "EXCHANGE_APP_ID setting should not be empty when using OpenExchangeBackend"
            )

        self.url = "{}?app_id={}&base={}".format(EXCHANGE_URL, EXCHANGE_APP_ID, EXCHANGE_BASE_CURRENCY)

    def update_rates(self):
        """
        Creates or updates rates for a source
        """
        rates = self.get_rates()
        for currency, value in six.iteritems(rates):
            try:
                rate = Rate.objects.get(currency=currency)
            except Rate.DoesNotExist:
                rate = Rate(currency=currency)

            rate.value = value
            rate.save()

    def get_rates(self):
        try:
            logger.debug("Connecting to url %s" % self.url)
            data = urlopen(self.url).read().decode("utf-8")
            return json.loads(data)['rates']

        except Exception as e:
            logger.exception("Error retrieving data from %s", self.url)
            raise RateBackendError("Error retrieving rates: %s" % e)



import datetime
from unittest import mock

from currency_open_exchange.models import Rate
from currency_open_exchange.utils import convert, get_rate, CURRENCIES
from decimal import Decimal
from django.test import SimpleTestCase


class ConversionTest(SimpleTestCase):

    def get_rate_mock(self, currency):
        if currency == 'USD':
            return self.usd_rate.value
        if currency == 'CNY':
            return self.cny_rate.value

    def setUp(self):
        self.usd_rate = mock.MagicMock(spec=Rate, value=1.000000, currency='USD')
        self.cny_rate = mock.MagicMock(spec=Rate, value=6.420400, currency='CNY')

    @mock.patch('currency_open_exchange.utils.get_rate')
    def test_convert_to_currency_ok(self, rate):
        expected_result = Decimal('64.20'), 'CNY'
        rate.side_effect = self.get_rate_mock
        result = convert(10, 'USD', 'CNY')

        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.get_rate')
    def test_convert_to_default_currency_ok(self, rate):
        expected_result = Decimal('15.58'), 'USD'
        rate.side_effect = self.get_rate_mock
        result = convert(100, 'CNY', 'USD')

        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.logger')
    @mock.patch('currency_open_exchange.utils.get_rate')
    def test_wrong_convert_from_currency(self, rate, logger):
        expected_result = Decimal('100'), 'WRONG'
        rate.side_effect = self.get_rate_mock
        result = convert(100, 'WRONG', 'USD')

        logger.error.assert_called_with('Cannot covert currency from WRONG to USD')
        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.logger')
    @mock.patch('currency_open_exchange.utils.get_rate')
    def test_wrong_convert_to_currency(self, rate, logger):
        expected_result = Decimal('100'), 'USD'
        rate.side_effect = self.get_rate_mock
        result = convert(100, 'USD', 'WRONG')

        logger.error.assert_called_with('Cannot covert currency from USD to WRONG')
        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.Rate')
    def test_get_rate_ok(self, rate):
        expected_result = 6.420400
        rate.objects.filter().first.return_value = self.cny_rate
        result = get_rate('CNY')

        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.CURRENCIES', {})
    @mock.patch('currency_open_exchange.utils.Rate')
    def test_get_rate_does_no_exist(self, rate):
        expected_result = None
        rate.objects.filter().first.return_value = None
        result = get_rate('WRONG')

        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.CURRENCIES', {'CNY': (str(datetime.date.today()), 6.420400)})
    @mock.patch('currency_open_exchange.utils.Rate')
    def test_get_rate_cached_ok(self, rate):
        expected_result = 6.420400
        result = get_rate('CNY')

        rate.objects.filter().first.assert_not_called()
        self.assertEqual(result, expected_result)

    @mock.patch('currency_open_exchange.utils.Rate')
    def test_get_rate_cache_updated_ok(self, rate):
        expected_result = {'CNY': (str(datetime.date.today()), 6.4204)}
        rate.objects.filter().first.return_value = self.cny_rate
        get_rate('CNY')

        self.assertEqual(CURRENCIES, expected_result)



from django.conf import settings

SECONDS_IN_DAY = 60*60*24

DEFAULT_BACKEND = 'djmoney_rates.backends.OpenExchangeBackend'
EXCHANGE_URL = 'http://openexchangerates.org/api/latest.json'
EXCHANGE_BASE_CURRENCY = getattr(settings, 'DJANGO_MONEY_RATES', 'USD')
EXCHANGE_APP_ID = getattr(settings, 'EXCHANGE_APP_ID')



import datetime
import logging
from decimal import Decimal
from .models import Rate

logger = logging.getLogger(__name__)

CURRENCIES = {}


def get_rate(currency):
    date = str(datetime.date.today())
    if currency in CURRENCIES and len(CURRENCIES[currency]) == 2 and CURRENCIES[currency][0] == date:
        return CURRENCIES[currency][1]

    rate = Rate.objects.filter(currency=currency).first()
    if not rate:
        return

    CURRENCIES[currency] = (date, rate.value)
    return rate.value


def base_convert_money(amount, rate_from, rate_to):
    """
    Convert 'amount' from 'currency_from' to 'currency_to'
    """
    return ((get_decimal(amount) / get_decimal(rate_from)) * get_decimal(rate_to)).quantize(Decimal("1.00"))


def get_decimal(value):
    return Decimal(value).quantize(Decimal('.000001'))


def convert(amount, currency_from, currency_to):
    """
    Convert 'amount' from 'currency_from' to 'currency_to' and return a Money
    instance of the converted amount.
    """
    rate_from = get_rate(currency_from)
    rate_to = get_rate(currency_to)

    if not rate_from or not rate_to:
        logger.error('Cannot covert currency from {} to {}'.format(currency_from, currency_to))
        return amount, currency_from

    new_amount = base_convert_money(amount, rate_from, rate_to)
    return new_amount, currency_to



from django.db import models
from django.utils.translation import ugettext_lazy as _


class Rate(models.Model):

    currency = models.CharField(max_length=3, unique=True)
    value = models.DecimalField(max_digits=20, decimal_places=6)
    updated = models.DateField(auto_now=True)

    class Meta:
        ordering = ('currency',)

    def __str__(self):
        return _("%s at %.2f") % (self.currency, self.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os
import re


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.match("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('currency_open_exchange')


LONG_DESCRIPTION = open('README.rst').read()

setup(
    name='currency_open_exchange',
    version=version,
    description='Currency conversion',
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 1.10",
        "Framework :: Django :: 1.11",
    ],
    keywords='currency-open-exchange',
    author='Anton Maistrenko',
    author_email='it2015maistrenko@gmail.com',
    url='https://github.com/MaistrenkoAnton/Currency-Open-Exchange',
    license="BSD",
    packages=find_packages(),
    include_package_data=True,
    test_suite='test',
    install_requires=[],
    zip_safe=False
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-05-27 11:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Rate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.CharField(max_length=3, unique=True)),
                ('value', models.DecimalField(decimal_places=6, max_digits=20)),
                ('updated', models.DateField(auto_now=True)),
            ],
            options={
                'ordering': ('currency',),
            },
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

from ...backends import RateBackend


class Command(BaseCommand):
    help = 'Update rates'

    def handle(self, *args, **options):
        try:
            RateBackend().update_rates()
        except Exception as e:
            raise CommandError("Error during rate update: %s" % e)

        self.stdout.write('Successfully updated rates')
